/***************************************************************************
  playlist.cpp  -  FlexPlaylists Noatun PlayList Object
  --------------------------------------------------------
  Maintainer: Stefan Gehn <sgehn@gmx.net>

  Copyright (c) 2000-2001 Charles Samuels <charles@kde.org>
                2000-2001 Neil Stevens <neil@qualityassistant.com>
                2003      Stefan Gehn <sgehn@gmx.net>

  Copyright (c) from the patches of (not sure if they are still contained in this code):
                2001 Klas Kalass <klas.kalass@gmx.de>
                2001 Anno v. Heimburg <doktor.dos@gmx.de>
 ***************************************************************************/


#include <qdragobject.h>
#include <qheader.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qmap.h>
#include <qregexp.h>
#include <qtextstream.h>

#include <kaction.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kpopupmenu.h>
#include <ksimpleconfig.h>
#include <kstandarddirs.h>
#include <kstdaction.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>
#include <kurldrag.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <kwin.h>
#include <kpropertiesdialog.h>

#include <qpainter.h>

#include <noatun/app.h>
#include <noatun/player.h>
#include <noatun/playlistsaver.h>
#include <noatun/stdaction.h>

#include "playlist.h"
#include "view.h"
#include "find.h"
#include "filejump.h"

// -----------------------------------------------------------------------------

//#define FPL_DEBUG 1 // my internal debug flag, I'm lazy :)

#define STATUS_CURLEN 1
#define STATUS_LISTLEN 2
#define STATUS_ITEMCOUNT 3

#if NOATUN_2_3
#define PLAYLISTFILE "flexplaylist.m3u"
#else
#define PLAYLISTFILE "flexplaylist.xml"
#endif

// -----------------------------------------------------------------------------

SafeListViewItem::SafeListViewItem(QListView *parent, QListViewItem *after, const KURL &text)
	: KListViewItem(parent, 0), PlaylistItemData(), removed(false)
{
	addRef();
	setUrl(text);

	//static_cast<KListView*>(parent)->moveItem(this, 0, after);
	moveItem(after);

	// is this really needed, it makes the listview too wide for me :(
//	setText(0,text.filename());

	/*
	FlexPlaylist *p = FPL;
	if (!(p->currentItem || p->nextItem || p->previousItem))
		p->setCurrent(this);
	if (p->currentItem==(SafeListViewItem*)itemAbove())
		p->setNext(this);
	if (p->currentItem==(SafeListViewItem*)itemBelow())
		p->setPrevious(this);
	*/

	if (!streamable() && enqueue(url()))
		setUrl(KURL(localFilename()));

	PlaylistItemData::added();
}

SafeListViewItem::SafeListViewItem(QListView *parent, QListViewItem *after, const QMap<QString,QString> &props)
	: KListViewItem(parent, 0), removed(false)
{
	addRef();

	// A version of setProperty that assumes a key is unique,
	// and doesn't call modified for every new key.
	// Ugly, but this function is a very hot path on playlist loading
	for (QMap<QString,QString>::ConstIterator i=props.begin(); i!=props.end(); ++i)
	{
		QString n = i.key();
		QString val = i.data();

		if (n!="enabled") // compatibility to splitplaylist
		{
			Property p={n,val};
			mProperties += p;
		}
	}

	//static_cast<KListView*>(parent)->moveItem(this, 0, after);
	moveItem(after);
	modified();

	/*
	FlexPlaylist *p=FPL;
	if (!(p->currentItem || p->nextItem || p->previousItem))
		p->setCurrent(this);
	if (p->currentItem==(SafeListViewItem*)itemAbove())
		p->setNext(this);
	if (p->currentItem==(SafeListViewItem*)itemBelow())
		p->setPrevious(this);
	*/

	if (!streamable() && enqueue(url()))
	{
		KURL u;
		u.setPath(localFilename());
		setUrl(u);
	}
	PlaylistItemData::added();
}

SafeListViewItem::~SafeListViewItem()
{
	remove();
}

QString SafeListViewItem::file() const
{
	return localFilename();
}

static void pad(QString &str)
{
	int len=str.length();
	int at = 0;
	int blocklen=0;

	static const int paddingsize=12;

	// not static for reason
	const QChar chars[paddingsize] =
	{
		QChar('0'), QChar('0'), QChar('0'), QChar('0'),
		QChar('0'), QChar('0'), QChar('0'), QChar('0'),
		QChar('0'), QChar('0'), QChar('0'), QChar('0')
	};

	for (int i=0; i < len; i++)
	{
		if (str[i].isNumber())
		{
			if (!blocklen)
				at = i;
			blocklen++;
		}
		else if (blocklen)
		{
			int pads=paddingsize;
			pads -= blocklen;
			str.insert(at, chars, pads);
			i += pads;
			blocklen = 0;
		}
	}
	if (blocklen)
	{
		int pads=paddingsize;
		pads -= blocklen;
		str.insert(at, chars, pads);
	}
}

int SafeListViewItem::compare(QListViewItem * i, int col, bool) const
{
	QString text1 = text(col);
	QString text2 = i->text(col);

	pad(text1);
	pad(text2);
	return text1.compare(text2);
}

QString SafeListViewItem::property(const QString &n, const QString &def) const
{
	for (QValueList<Property>::ConstIterator i=mProperties.begin(); i != mProperties.end(); ++i)
	{
		if ((*i).key==n)
			return (*i).value;
	}
	return def;
}

void SafeListViewItem::setProperty(const QString &n, const QString &val)
{
	if(n=="enabled") // splitplaylist compat
		return;

	if(property(n,"") == val)
	{
//			kdDebug(66666) << "SafeListViewItem::setProperty(), property unchanged!" << endl;
		return;
	}

	clearProperty(n);
	Property p={n,val};
	mProperties += p;

	modified();
}

void SafeListViewItem::clearProperty(const QString &n)
{
	if (n=="enabled") // splitplaylist compat
		return;

	for (QValueList<Property>::Iterator i=mProperties.begin(); i != mProperties.end(); ++i)
	{
		if ((*i).key==n)
		{
			mProperties.remove(i);
			modified();
			break;
		}
	}
}

QStringList SafeListViewItem::properties() const
{
	QStringList list;
	for (QValueList<Property>::ConstIterator i=mProperties.begin(); i != mProperties.end(); ++i)
		list += (*i).key;

	return list;
}

bool SafeListViewItem::isProperty(const QString &n) const
{
	for (QValueList<Property>::ConstIterator i=mProperties.begin(); i != mProperties.end(); ++i)
	{
		if ((*i).key==n)
			return true;
	}

	return false;
}

void SafeListViewItem::downloaded(int percent)
{
	if (!removed)
		setText(TIME_COL, QString::number(percent)+'%');
}

void SafeListViewItem::downloadTimeout()
{
	setText(TIME_COL, "-");
}

void SafeListViewItem::downloadFinished()
{
	setText(TIME_COL, "");
}

void SafeListViewItem::modified()
{
	//bool widthChangeNeeded = false;

	if (text(TITLE_COL)!=title())
	{
		setText(TITLE_COL, title());
		updatePixmap(); // new text probably means new file and thus changed mimetype
		//widthChangeNeeded = true;
	}

	if (isDownloaded() && length()!=-1 && text(TIME_COL)!=lengthString())
	{
		QString oldTime = text(TIME_COL);
		QString newTime = lengthString();
		FPL->lengthChanged(oldTime, newTime);

		setText(TIME_COL, lengthString());
		//widthChangeNeeded = true;
	}

//	if (widthChangeNeeded)
//		widthChanged(-1);

	PlaylistItemData::modified();
}

void SafeListViewItem::updatePixmap()
{
	if (FPL->prefs()->showIcons())
		setPixmap(TITLE_COL, KMimeType::pixmapForURL(url(), 0, KIcon::Small));
	else
	{
//		kdDebug(66666) << k_funcinfo << "setting empty pixmap" << endl;
		setPixmap(TITLE_COL, QPixmap());
	}
}

void SafeListViewItem::remove()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;

	removed=true;
	if (napp->player()->current()==this)
	{
//		kdDebug(66666) << k_funcinfo << "removed currentItem!" << endl;

		if (!itemAbove() && !itemBelow())
		{
			napp->player()->stop();
			FPL->setCurrent(0);
		}
		else
		{
			bool wasStopped = napp->player()->isStopped();

			//FPL->setCurrent(0);
			if (FPL->exiting())
			{
				FPL->setCurrent(0);
				return;
			}

			// TODO: This code was taken from player()->forward(), maybe fix noatun instead?
			napp->player()->stop();
			if (FPL->next())
			{
				/*kdDebug(66666) << k_funcinfo <<
					"found next, was paused or playing, calling play()" << endl;*/
				if (!wasStopped)
					napp->player()->play();
			}
			else if (napp->loopList())
			{
				//kdDebug(66666) << k_funcinfo << "restarting from first item" << endl;
				if (FPL->reset(), FPL->current())
				{
					//kdDebug(66666) << k_funcinfo << "was paused or playing, calling play()" << endl;
					if(!wasStopped)
						napp->player()->play();
				}
			}
			//napp->player()->forward();
		}
	}

	if (listView())
	{
		if (FPL->currentItem==this)
			FPL->setCurrent(static_cast<SafeListViewItem*>(itemBelow()));

		listView()->takeItem(this);
	}
	else if (FPL->currentItem==this)
	{
		FPL->setCurrent(0);
	}

	PlaylistItemData::removed();
}

void SafeListViewItem::paintCell(QPainter *p, const QColorGroup &cg,
	int column, int width, int alignment)
{
	if(FPL->currentItem == this)
	{
		QColorGroup _cg(cg);
		QColor oldTextColor = _cg.text();
		QColor oldBgColor = _cg.text();
		QFont oldFont = p->font();

		_cg.setColor(QColorGroup::Text, FPL->prefs()->selTxtColor());
		_cg.setColor(QColorGroup::Base, FPL->prefs()->selBgColor());

		if (FPL->prefs()->selUseBold())
		{
			QFont newFont(oldFont);
			newFont.setBold(true);
			p->setFont(newFont);
		}
		QListViewItem::paintCell(p, _cg, column, width, alignment);

		// restore old values (just for safety)
		p->setFont(oldFont);
		_cg.setColor(QColorGroup::Text, oldTextColor);
		_cg.setColor(QColorGroup::Base, oldBgColor);
	}
	else
		KListViewItem::paintCell(p, cg, column, width, alignment);
}
/*
int SafeListViewItem::width(const QFontMetrics &fm, const QListView *lv, int c) const
{
	if((FPL->currentItem == this) && FPL->prefs()->selUseBold())
	{
		QFont newFont(FPL->prefs()->font());
		newFont.setBold(true);
		QFontMetrics newMetrics(newFont);
		return newMetrics.width((text(c)));
	}

	return KListViewItem::width(fm,lv,c);
}*/

// --------------------------------------------------------------------------------------

List::List(View *parent) : KListView(parent, "List"),
	recursiveAddAfter(0),
	listJob(0),
	mDropMenu(0),
	dropAfterItem(0)
{
	// FIXME: Is this needed? After all paintEvent() is not done by ourselves like it was in SPL
//	setWFlags(Qt::WStaticContents|Qt::WRepaintNoErase);

	addColumn(i18n("Name"),10);
	addColumn(i18n("Time"),10);
	setSorting(-1);
	setAcceptDrops(true);
	setDropVisualizer(true);
	setDragEnabled(true);
	setItemsMovable(true);
	setSelectionMode(QListView::Extended);
	setShowSortIndicator(true);
	setHScrollBarMode(AlwaysOff);

	slotUpdateColumnWidths();

	connect(this, SIGNAL(dropped(QDropEvent*, QListViewItem*)),
		this, SLOT(dropEvent(QDropEvent*, QListViewItem*)));
	/*connect(this, SIGNAL(moved(QPtrList<QListViewItem>&,QPtrList<QListViewItem>&,QPtrList<QListViewItem>&)),
		this, SLOT(move(QPtrList<QListViewItem>&,QPtrList<QListViewItem>&,QPtrList<QListViewItem>&)));*/
	connect(this, SIGNAL(moved()),
		this, SLOT(move()));
	connect(this, SIGNAL(aboutToMove()),
		parent, SLOT(setNoSorting()));
	connect(this, SIGNAL(deleteCurrentItem()),
		this, SLOT(deleteSelected()));
	connect(this, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)),
		parent, SLOT(showItemContextMenu(KListView*, QListViewItem*, const QPoint&)));
}

List::~List()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
}

/*void List::move(QPtrList<QListViewItem>& item,
	QPtrList<QListViewItem>&, QPtrList<QListViewItem>&)
{
	move(item);
}

void List::move(QPtrList<QListViewItem>& item)
*/

void List::move()
{
/*	bool bidimerge = static_cast<bool>(item.containsRef(
		static_cast<SafeListViewItem*>(FPL->currentItem.data())));

	bool current = FPL->currentItem;

	if (current)
	{
		if (bidimerge || item.containsRef(static_cast<SafeListViewItem*>(FPL->previousItem.data()))
		              || (current && item.containsRef(static_cast<SafeListViewItem*>(FPL->currentItem.data())->itemAbove())))
			FPL->setPrevious(static_cast<SafeListViewItem*>(static_cast<SafeListViewItem*>(FPL->currentItem.data())->itemAbove()));

		if (bidimerge || item.containsRef(static_cast<SafeListViewItem*>(FPL->nextItem.data()))
		              || item.containsRef(static_cast<SafeListViewItem*>(static_cast<SafeListViewItem*>(FPL->currentItem.data())->itemBelow())))
			FPL->setNext(static_cast<SafeListViewItem*>(static_cast<SafeListViewItem*>(FPL->currentItem.data())->itemBelow()));
	}
*/
	emit modified();
}

bool List::acceptDrag(QDropEvent *event) const
{
	return KURLDrag::canDecode(event) || KListView::acceptDrag(event);
}

void List::dropEvent(QDropEvent *event, QListViewItem *after)
{
	dropItemList.clear();
	if (!KURLDrag::decode(event, dropItemList))
		return;

	if (dropItemList.isEmpty())
		return;

	dropAfterItem = after;

	if (mDropMenu==0)
	{
		mDropMenu = new KPopupMenu(this);
		mDropMenu->insertItem(i18n("Insert here"), 0);
		mDropMenu->insertItem(i18n("Add to end of playlist"), 1);
		mDropMenu->insertItem(i18n("Cancel"), 2);
		connect(mDropMenu, SIGNAL(activated(int)), SLOT(slotDropMenuActivated(int)));
	};

	mDropMenu->popup(mapToGlobal(contentsToViewport(event->pos())), 0);
}

void List::slotDropMenuActivated(int item)
{
	QListViewItem *lastone;
	KURL::List::Iterator i;

	switch(item)
	{
		case 0: // insert here
		{
			lastone = dropAfterItem; // add items at selected place in playlist
			static_cast<View*>(parent())->setNoSorting();

			for (i=dropItemList.begin(); i != dropItemList.end(); ++i)
				lastone = addFile(*i, false, lastone);

			emit modified();
			break;
		}
		case 1:
		{
			lastone = lastItem(); // add items at end of playlist
			static_cast<View*>(parent())->setNoSorting();

			for (i=dropItemList.begin(); i != dropItemList.end(); ++i)
				lastone = addFile(*i, false, lastone);

			emit modified();
			break;
		}
		case 2:
			break;
	}
}

void List::keyPressEvent(QKeyEvent *e)
{
//	kdDebug(66666) << "flexplaylist: List::keyPressEvent(QKeyEvent *e); key=" << e->key() << endl;

	if ((e->key()==Key_Enter || e->key()==Key_Return) && e->state() == 0)
	{
		if (currentItem())
			emit KListView::executed(currentItem());
		return;
	}

	if (e->key()==Key_Delete)
	{
		if (currentItem())
			emit deleteCurrentItem();
		return;
	}

	KListView::keyPressEvent(e);
}

void List::viewportResizeEvent(QResizeEvent *e)
{
	// If the width of the view has changed, manually update the column
	// widths.
	if(e->size().width() != e->oldSize().width())
		slotUpdateColumnWidths();

	KListView::viewportResizeEvent(e);
}

void List::slotUpdateColumnWidths()
{
	QFont newFont(FPL->prefs()->font());
	newFont.setBold(true);
	QFontMetrics newMetrics(newFont);

	int timeWidth = 10;
	for (SafeListViewItem *i=static_cast<SafeListViewItem*>(firstChild());
	     i != 0; i=static_cast<SafeListViewItem*>(i->itemBelow()))
	{
		int neededWidth = newMetrics.width(i->text(TIME_COL))+2;
		if(neededWidth > timeWidth)
			timeWidth = neededWidth;
	}
	timeWidth += 2;
	setColumnWidth(TIME_COL, timeWidth);
	setColumnWidth(TITLE_COL, visibleWidth() - timeWidth);
}

// Use this only once!!!
class NoatunSaver : public PlaylistSaver
{
	List *mList;
	SafeListViewItem *after;
public:
	NoatunSaver(List *l, QListViewItem *after=0)
		: mList(l)
	{
		this->after= static_cast<SafeListViewItem*>(after);
	}

	QListViewItem *getAfter() { return after;}

protected:
	virtual void readItem(const QMap<QString,QString> &properties)
	{
//		kdDebug(66666) << k_funcinfo << "url='" << properties["url"] << "'" << endl;
		after=new SafeListViewItem(mList, after, properties);
	}

	virtual PlaylistItem writeItem()
	{
		if (!after)
			after=static_cast<SafeListViewItem*>(mList->firstChild());
		else
			after=static_cast<SafeListViewItem*>(static_cast<SafeListViewItem*>(after)->nextSibling());
		return PlaylistItem(after);
	}
};


bool View::saveToURL(const KURL &url)
{
#if NOATUN_2_3
	int format=PlaylistSaver::EXTM3U; // default for new kde 3.2 noatun
#else
	int format=PlaylistSaver::XMLPlaylist; // default for kde 3.1
#endif

	QString end=url.filename().right(3).lower();
	if (end=="pls") // FIXME: This should happen in PlaylistSaver::save()
		format=PlaylistSaver::PLS;
//	else if (end=="wsx" || end=="asx") // FIXME: This should happen in PlaylistSaver::save()
//		format=PlaylistSaver::ASX;
	else if (end!="m3u") // something else, use PlaylistSaver default
		format=0;

//	kdDebug(66666) << k_funcinfo << "called. file ext is '" << end << "', int format=" << format << endl;

	NoatunSaver saver(list);
	if(saver.save(url,format))
	{
		return true;
	}
	else
	{
		KMessageBox::error(this, i18n("Couldnotwriteto%1.").arg(url.prettyURL()));
		return false;
	}
}

void View::exportTo(const KURL &url)
{
	QString local(napp->tempSaveName(url.path()));
	QFile saver(local);
	saver.open(IO_ReadWrite | IO_Truncate);
	QTextStream t(&saver);
	// navigate the list
	for (SafeListViewItem *i=static_cast<SafeListViewItem*>(listView()->firstChild());
	     i != 0; i=static_cast<SafeListViewItem*>(i->itemBelow()))
	{
		KURL u=i->url();
		if (u.isLocalFile())
			t << u.path() << '\n';
		else
			t << u.url() << '\n';
	}
	saver.close();

#if KDE_VERSION < KDE_MAKE_VERSION(3, 1, 90)
	KIO::NetAccess::upload(local, url);
#else
	KIO::NetAccess::upload(local, url, this);
#endif

	saver.remove();
}

QListViewItem *List::openGlobal(const KURL &u, QListViewItem *after)
{
//	kdDebug(66666) << k_funcinfo << endl;

	clear();
	NoatunSaver saver(this, after);
	saver.metalist(u);

	return saver.getAfter();
}

// for m3u files
QListViewItem *List::importGlobal(const KURL &url, QListViewItem *after)
{
//	kdDebug(66666) << k_funcinfo << "url='" << url.url() << "'" << endl;

	NoatunSaver saver(this, after);
	if (!saver.metalist(url))
	{
//		kdDebug(66666) << "no valid playlist" << endl;
		after=new SafeListViewItem(this, after, url);
//		FPL->listItemSelected(after);
		return after;
	}
	return saver.getAfter();
}

QListViewItem *List::addFile(const KURL& url, bool play, QListViewItem *after)
{
//	kdDebug(66666) << k_funcinfo << "url='" << url.url() << "', play=" << play << endl;

	// when a new item is added, we don't want to sort anymore
	FPL->view->setNoSorting();

	QString ext = url.path().right(3).lower();

#if NOATUN_2_3
	if(ext=="m3u" ||
		ext=="xml" ||
		ext=="pls" ||
		ext=="wsx" ||
		ext=="wvx" ||
		ext=="asx" ||
		url.protocol().lower()=="http")
#else
	if(ext=="m3u" ||
		ext=="xml" ||
		ext=="pls" ||
		url.protocol().lower()=="http")
#endif
	{
//		kdDebug(66666) << "adding playlist..." << endl;
		// a playlist is requested
		return importGlobal(url, after);
	}
	else
	{
//		kdDebug(66666) << "adding normal file..." << endl;
		if (!after)
			after = lastItem();

		KFileItem fileItem(KFileItem::Unknown,KFileItem::Unknown,url);
		if (fileItem.isDir())
		{
			addDirectoryRecursive(url, after);
			return after; // FIXME: don't (and can't) know better!?
		}
		else
		{
			QListViewItem *i=new SafeListViewItem(this, after, url);
			if (play)
				FPL->listItemSelected(i);
			return i;
		}
	}
}

// starts a new listJob if there is no active but work to do
void List::addNextPendingDirectory()
{
	KURL::List::Iterator pendingIt= pendingAddDirectories.begin();
	if (!listJob && (pendingIt!=pendingAddDirectories.end()))
	{
		currentJobURL= *pendingIt;
		listJob = KIO::listRecursive(currentJobURL, false, false);
		connect(
				listJob, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList&)),
				SLOT(slotEntries(KIO::Job*, const KIO::UDSEntryList&))
			);
		connect(
				listJob, SIGNAL(result(KIO::Job *)),
				SLOT(slotResult(KIO::Job *))
			);
		connect(
				listJob, SIGNAL(redirection(KIO::Job *, const KURL &)),
				SLOT(slotRedirection(KIO::Job *, const KURL &))
			);
		pendingAddDirectories.remove(pendingIt);
	}
}

void List::addDirectoryRecursive(const KURL &dir, QListViewItem *after)
{
	if (!after) after=lastItem();
	recursiveAddAfter= after;
	pendingAddDirectories.append(dir);
	addNextPendingDirectory();
}

void List::slotResult(KIO::Job *job)
{
	listJob = 0;
	if (job && job->error())
		job->showErrorDialog();
	addNextPendingDirectory();
}

void List::slotEntries(KIO::Job *, const KIO::UDSEntryList &entries)
{
	QMap<QString,KURL> __list; // temp list to sort entries

	KIO::UDSEntryListConstIterator it = entries.begin();
	KIO::UDSEntryListConstIterator end = entries.end();

	for (; it != end; ++it)
	{
		KFileItem file(*it, currentJobURL, false /* no mimetype detection */, true);
		// "prudhomm:
		// insert the path + url in the map to sort automatically by path
		// note also that you use audiocd to rip your CDs then it will be sorted the right way
		// now it is an easy fix to have a nice sort BUT it is not the best
		// we should sort based on the tracknumber"
		// - copied over from old kdirlister hack <hans_meine@gmx.de>
		if (!file.isDir())
			__list.insert(file.url().path(), file.url());
	}
	QMap<QString,KURL>::Iterator __it;
	for(__it = __list.begin(); __it != __list.end(); ++__it)
	{
		recursiveAddAfter = addFile(__it.data(), false, recursiveAddAfter);
	}
}

void List::slotRedirection(KIO::Job *, const KURL & url)
{
	currentJobURL = url;
}

// ==============================================================================

View::View(const char *name) : KMainWindow(0, name)
{
//	kdDebug(66666) << k_funcinfo << "Called" << endl;

//	KWin::setType(winId(), NET::Toolbar); // breaks focusing after fileselectors

	list = new List(this);
	setCentralWidget(list);

	mFinder = new Finder(this);
	connect(mFinder, SIGNAL(search(Finder*)), this, SLOT(findIt(Finder*)));

	mFileJumpDialog = new FileJumpDialog(this);
	connect(mFileJumpDialog, SIGNAL(doJump(const PlaylistItem&)),
		this, SLOT(slotDoJump(const PlaylistItem&)));

	// ------------------------------------------------------------------------
	statusBar()->insertItem("00:00", STATUS_CURLEN, 0, true);
	statusBar()->insertItem("00:00", STATUS_LISTLEN, 0, true);
//#ifdef FPL_DEBUG
	statusBar()->insertItem("0", STATUS_ITEMCOUNT, 0, true);
//#endif

	mStatusBarWidget = new QHBox(statusBar(), "mStatusBarWidget");
	mStatusBarWidget->setSpacing(0);
	mStatusBarWidget->setMargin(0);
	statusBar()->addWidget(mStatusBarWidget, 0, true);

	mPlayPauseButton = new TrayButton(mStatusBarWidget,"playpause");
	mPlayPauseButton->setPixmap(SmallIcon("noatunplay"));
	mPlayPauseButton->setFixedSize(18,18);
	connect(mPlayPauseButton, SIGNAL(clicked()), napp->player(), SLOT(playpause()));

	TrayButton *btn;
	btn = new TrayButton(mStatusBarWidget,"back");
	btn->setPixmap(SmallIcon("noatunback"));
	btn->setFixedSize(18,18);
	connect(btn, SIGNAL(clicked()), napp->player(), SLOT(back()));

	mStopButton = new TrayButton(mStatusBarWidget,"stop");
	mStopButton->setPixmap(SmallIcon("noatunstop"));
	mStopButton->setFixedSize(18,18);
	connect(mStopButton, SIGNAL(clicked()), napp->player(), SLOT(stop()));

	btn = new TrayButton(mStatusBarWidget,"forward");
	btn->setPixmap(SmallIcon("noatunforward"));
	btn->setFixedSize(18,18);
	connect(btn, SIGNAL(clicked()), napp->player(), SLOT(forward()));

	// sets initial status for playbutton and stopbutton
	if (napp->player()->isPlaying())
		playing();
	else if (napp->player()->isPaused() || napp->player()->isStopped())
		notplaying();
	connect(napp->player(), SIGNAL(playing()), this, SLOT(playing()));
	connect(napp->player(), SIGNAL(paused()), this, SLOT(notplaying()));
	connect(napp->player(), SIGNAL(stopped()), this, SLOT(notplaying()));
	// ------------------------------------------------------------------------

	connect(list, SIGNAL(modified(void)), this, SLOT(setModified(void)));
	// connect the click on the header with sorting
	connect(list->header(),SIGNAL(clicked(int)),this,SLOT(headerClicked(int)));
	connect(list, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));

	connect(napp->player(), SIGNAL(newSong()), this, SLOT(updateStatusBar()));

	// List related actions -----------------------------------------------------
	mOpen = new KAction(i18n("&Add Files..."), "add_file", Qt::CTRL+Qt::Key_Plus,
		this, SLOT(addFiles()), actionCollection(), "add_files");
	(void) new KAction(i18n("Add &Folders..."), "add_dir", 0,
		this, SLOT(addDirectory()),  actionCollection(), "add_dir");
	mClose = KStdAction::close(this, SLOT(close()), actionCollection());
	mFind = KStdAction::find(this, SLOT(find()), actionCollection(), "find");

	mJump = new KAction(i18n("Jump To File..."), "goto", Qt::CTRL+Qt::Key_J,
		this, SLOT(jumpTo()), actionCollection(), "jumpto");

	(void) new KAction(i18n("&Shuffle"), "misc", Qt::CTRL+Qt::Key_Slash, FPL,
		SLOT(randomize()), actionCollection(), "shuffle");
	(void) new KAction(KStdGuiItem::clear(), 0, list,
		SLOT(clear()), actionCollection(), "clear");

	// Item related actions -----------------------------------------------------
	mPlayCurrent = new KAction(i18n("&Play"), "player_play", Qt::Key_Return,
		this, SLOT(playCurrent()),  actionCollection(), "item_play");
	mPlaySelectedNext = new KAction(i18n("Play This &Next"), 0, 0,
		list, SLOT(moveSelectionAfterCurrent()), actionCollection(), "item_playnext");
	mDelete = new KAction(i18n("Delete"), 0, Key_Delete,
		list, SLOT(deleteSelected()),  actionCollection(), "item_delete");
	mDelete = new KAction(i18n("Remove from Disk"), "editdelete", 0,
		list, SLOT(removeFromDisk()),  actionCollection(), "item_remove");
	mItemProperties = new KAction(i18n("P&roperties..."), "info", Qt::ALT+Qt::Key_Return,
		this, SLOT(showItemProperties()), actionCollection(), "item_properties");

	// Config related actions ---------------------------------------------------
	(void) KStdAction::configureToolbars(this, SLOT(configureToolBars()),
		actionCollection());
	menubarAction = KStdAction::showMenubar(this, SLOT(showMenubar()),
		actionCollection());
	statusbarAction = KStdAction::showStatusbar(this, SLOT(showStatusbar()),
		actionCollection());
	KStdAction::keyBindings(this, SLOT(slotConfKeys()), actionCollection());

	KStdAction::preferences(this, SLOT(slotOpenPrefs()),
		actionCollection(), "settings_preferences");

	// Player related actions ---------------------------------------------------
	mOpenNew=KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
	mOpenpl = KStdAction::open(this, SLOT(open()), actionCollection());
	mSave = KStdAction::save(this, SLOT(save()), actionCollection());
	mSaveAs = KStdAction::saveAs(this, SLOT(saveAs()), actionCollection());

	mBack = NoatunStdAction::back(actionCollection(), "player_back");
	mPlayPause = NoatunStdAction::playpause(actionCollection(), "player_playpause");
	mStop = NoatunStdAction::stop(actionCollection(), "player_stop");
	mForward = NoatunStdAction::forward(actionCollection(), "player_forward");
#if NOATUN_2_3
	mLoop = NoatunStdAction::loop(actionCollection(), "player_loop");
#endif

	setStandardToolBarMenuEnabled(true);

	createGUI("fplui.rc");

	applyMainWindowSettings(KGlobal::config(), "FPL Window");
	menubarAction->setChecked(!menuBar()->isHidden());
	statusbarAction->setChecked(!statusBar()->isHidden());

	slotLoadWindowState();
	slotSelectionChanged(); // enable/disable actions

	list->setFocus();
}

void View::showItemContextMenu(KListView*, QListViewItem* i, const QPoint& p)
{
	if (i)
	{
		QPopupMenu *pop = dynamic_cast<QPopupMenu *>(factory()->container("listpopup", this));
		if(pop)
			pop->popup(p);
	}
}

void View::playing()
{
	mPlayPauseButton->setPixmap(SmallIcon("noatunpause"));
	mStopButton->setEnabled(true);
}

void View::notplaying()
{
	mPlayPauseButton->setPixmap(SmallIcon("noatunplay"));
	if(napp->player()->isStopped())
		mStopButton->setEnabled(false);
	else if (napp->player()->isPaused())
		mStopButton->setEnabled(true);
}

void View::showMenubar()
{
	if(menubarAction->isChecked())
		menuBar()->show();
	else
		menuBar()->hide();
}

void View::showStatusbar()
{
	if(statusbarAction->isChecked())
		statusBar()->show();
	else
		statusBar()->hide();
}

void View::find()
{
	mFinder->show();
}


void View::jumpTo()
{
	mFileJumpDialog->show();
}


static bool testWord(QListViewItem *i, const QString &finder)
{
	PlaylistItemData *item=static_cast<SafeListViewItem*>(i);
	if (item->title().find(finder, 0, false) >=0)
		return true;
	if (item->file().find(finder, 0, false) >=0)
		return true;
	if (item->url().path().find(finder.local8Bit(), 0, false) >=0)
		return true;
	if (item->lengthString().find(finder, 0, false) >=0)
		return true;
	if (item->mimetype().find(finder.local8Bit(), 0, false) >=0)
		return true;
	return false;
}

static bool testWord(QListViewItem *i, const QRegExp &finder)
{
	PlaylistItemData *item=static_cast<SafeListViewItem*>(i);
	if (item->title().find(finder) >=0)
		return true;
	if (item->file().find(finder) >=0)
		return true;
	if (item->url().path().find(finder) >=0)
		return true;
	if (item->lengthString().find(finder) >=0)
		return true;
	if (item->mimetype().find(finder) >=0)
		return true;
	return false;
}

void View::findIt(Finder *f)
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;

	QListViewItem *search=list->currentItem();

	if (list->currentItem())
	{
		if (f->isForward())
			search=list->currentItem()->itemBelow();
		else
			search=list->currentItem()->itemAbove();
	}
	else
	{
		if (f->isForward())
			search=list->firstChild();
		else
			search=list->lastChild();
	}


	while (search)
	{
		if (f->regexp())
		{
			if (testWord(search, QRegExp(f->string(), false)))
				break;
		}
		else
		{
			if (testWord(search, f->string()))
				break;
		}

		if (f->isForward())
			search=search->itemBelow();
		else
			search=search->itemAbove();

		if (!search)
		{
			if (f->isForward())
			{
				if (KMessageBox::questionYesNo(this,
					i18n("End of playlist reached. Continue searching from beginning?")) == KMessageBox::Yes)
				{
					search=list->firstChild();
				}
			}
			else
			{
				if (KMessageBox::questionYesNo(this,
					i18n("Beginning of playlist reached. Continue searching from end?")) == KMessageBox::Yes)
				{
					search=list->lastChild();
				}
			}
		}
	}

	if (search)
	{
		list->clearSelection(); // unselect everything
		list->setSelected(search, true);
		list->setCurrentItem(search);
		list->ensureItemVisible(search);
	}
}

View::~View()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
	slotSaveWindowState();
	napp->player()->stop();
	//hide();
	saveState();
//	delete list;
}

void View::init()
{
	KURL internalURL;
	// see if we are importing an old-style list
#if NOATUN_2_3
	bool importing = !QFile(napp->dirs()->saveLocation("data", "noatun/") + "flexplaylist.m3u").exists();

	if (importing)
	{
		internalURL.setPath(napp->dirs()->saveLocation("data", "noatun/") + "flexplaylist.xml");
		NoatunSaver saver(list, 0);
		saver.load(internalURL, PlaylistSaver::XMLPlaylist);
	}
	else
	{
		internalURL.setPath(napp->dirs()->saveLocation("data", "noatun/") + "flexplaylist.m3u");
		list->openGlobal(internalURL);
	}
#else   // KDE 3.1 Noatun
		internalURL.setPath(napp->dirs()->saveLocation("data", "noatun/") + "flexplaylist.xml");
		list->openGlobal(internalURL);
#endif

	slotReadConfig();
	connect(FPL->prefs(), SIGNAL(configChanged()), this, SLOT(slotReadConfig()));

	// this has to come before setModified, since setModified uses the filename
	// to set the caption
	mPlaylistFile.setPath(FPL->prefs()->file());

	// this has to come after openGlobal, since openGlobal emits modified()
	setModified(FPL->prefs()->modified());
//	mPlaylistFile.setPath(FPL->prefs()->file());

	FPL->reset();
	int saved = FPL->prefs()->saved();

	PlaylistItem item = FPL->getFirst();
	for(int i = 0 ; i < saved ; i++)
		item = FPL->getAfter(item);

	if (item)
		FPL->setCurrent(item);
}

void View::save()
{
//	kdDebug(66666) << k_funcinfo << "Called" << endl;
	if (!mPlaylistFile.isValid())
	{
		saveAs();
		return;
	}

	if(saveToURL(mPlaylistFile))
		setModified(false);
}

void View::saveAs()
{
	KURL u=KFileDialog::getSaveURL(0, "Playlists(*.xml *.pls *.m3u *.asx *.wsx *.wvx)",
		this, i18n("Save Playlist"));
	if (!u.isValid())
		return;
	mPlaylistFile = u;
	save();
}

void View::open()
{
	if (saveModified())
	{
		KURL u = KFileDialog::getOpenURL(0,
			"Playlists(*.xml *.pls *.m3u *.asx *.wsx *.wvx)", this,
			i18n("Open Playlist"));

		if (!u.isValid())
			return;
		list->openGlobal(u);
		mPlaylistFile = u;
		setModified(false);
	}
}

void View::openNew()
{
	if (saveModified())
	{
		KURL u = KFileDialog::getOpenURL(0,
			"Playlists(*.xml *.pls *.m3u *.asx *.wsx *.wvx)", this,
			i18n("Open New Playlist"));

		if (!u.isValid())
			return;
		mPlaylistFile = u;

		listView()->clear();
		list->openGlobal(mPlaylistFile);
		mPlaylistFile = u;

		setModified(false);
	}
}

void View::fileNew()
{
	if (saveModified())
	{
		KURL internalURL;
		internalURL.setPath(
			napp->dirs()->saveLocation("data", "noatun/")+PLAYLISTFILE);
		mPlaylistFile = internalURL;
		listView()->clear();
		setModified(false);
	}
}

bool View::saveModified()
{
	bool completed = true;
	if (!modified)
		return completed;

	int want_save = KMessageBox::warningYesNoCancel(this,
			i18n("The current playlist has been modified.\n"
			     "Do you want to save it?"),
			i18n("Warning"));
	switch(want_save)
	{
		case KMessageBox::Yes:
		{
			if(mPlaylistFile.isValid())
			{
				saveToURL(mPlaylistFile);
				setModified(false);
			}
			else
			{
				saveAs();
			};

			completed = true;
			break;
		}
		case KMessageBox::No:
		{
			completed = true;
			break;
		}
		case KMessageBox::Cancel:
		{
			completed = false;
			break;
		}
		default:
		{
			completed = false;
			break;
		}
	}
	return completed;
}

void View::showItemProperties()
{
	QPtrList<QListViewItem> items(list->selectedItems());

	if (items.isEmpty())
	{
		KMessageBox::sorry(this, i18n("No item(s) selected."));
		return;
	}

	KFileItemList files;
	for (QListIterator<QListViewItem> it(items); it.current(); ++it)
	{
		KURL url = static_cast<SafeListViewItem*>(*it)->url();
		KFileItem *fileItem = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, url);
		if (fileItem->isReadable())
			files.append(fileItem);
	}
	if (!files.isEmpty())
		(void) new KPropertiesDialog(files, 0L, 0L, false, true);
	else
	{
		KMessageBox::sorry(this,
			i18n("Unable to read properties for selected item.",
			"Unable to read properties for selected items.", items.count()));
	}
}

void List::moveSelectionAfterCurrent()
{
	emit aboutToMove();

	QPtrList<QListViewItem> sel(selectedItems());
	QPtrListIterator<QListViewItem> it(sel);

	for (it.toLast(); it.current(); --it)
	{
		QListViewItem *item = it;
		//QListViewItem *parent = 0;
		QListViewItem *after = static_cast<SafeListViewItem*>(FPL->current().data());
		if (item != after)
		{
//			moveItem(item, parent, after);
			item->moveItem(after);
		}
	}
	for (it.toFirst(); it.current(); ++it)
	{
		QListViewItem *item = it;
		setSelected(item, true);
	}
	//move(sel);
	emit modified();
}

void List::removeFromDisk()
{
	QPtrList<QListViewItem> items(selectedItems());

	if (items.isEmpty())
	{
		KMessageBox::sorry(this, i18n("No item(s) selected."));
		return;
	}

	QStringList files;
	for (QListIterator<QListViewItem> it(items); it.current(); ++it)
	{
		KURL url = static_cast<SafeListViewItem*>(*it)->url();
		if(url.isLocalFile())
			files.append(url.path());
	}

	QString message;

	if(files.count() == 1)
	    message = i18n("Do you really want to delete this item from your disk?");
	else
	    message = i18n("Do you really want to delete these %1 items from your disk?").arg(QString::number(files.count()));

	if(KMessageBox::questionYesNoList(this, message, files) == KMessageBox::Yes)
	{
		bool mod = false;
		for (QListIterator<QListViewItem> it(items); it.current(); ++it)
		{
			KURL url = static_cast<SafeListViewItem*>(*it)->url();
			if(url.isLocalFile())
			{
				if(QFile::remove(url.path()))
				{
					static_cast<SafeListViewItem*>(*it)->remove();
					//delete *it;
					mod = true;
				}
				else
					KMessageBox::sorry(this, i18n("Could not delete ") + url.path() + ".");
			}
		}
		if(mod)
			emit modified();
	}
}


void View::playCurrent()
{
//	kdDebug(66666) << k_funcinfo << "called" << endl;
	listView()->keyPressEvent(new QKeyEvent(QEvent::KeyPress, Key_Enter, 13, 0));
}

void List::clear()
{
//	kdDebug(66666) << k_funcinfo << "called" << endl;
	FPL->setCurrent(0);
	QListView::clear();
}

void List::deleteSelected()
{
//	kdDebug(66666) << k_funcinfo << "called" << endl;

	QPtrList<QListViewItem> items(selectedItems());
	for (QPtrListIterator<QListViewItem> it(items); it.current(); ++it)
		static_cast<SafeListViewItem*>(*it)->remove();
	emit modified();
}

void View::addFiles()
{
	KURL::List files=KFileDialog::getOpenURLs(":mediadir", napp->mimeTypes(),
		this, i18n("Select File to Play"));

	if (files.count())
	{
		QListViewItem *last = list->lastItem();
		for(KURL::List::Iterator it=files.begin(); it!=files.end(); ++it)
			last = addFile(KURL(*it), false);
		setModified(true);
	}
}

void View::addDirectory()
{
	QString file=KFileDialog::getExistingDirectory(0, this, i18n("Select Folder"));

	if (file)
	{
		KURL url;
		url.setPath(file);
		list->addDirectoryRecursive(url);
		setModified(true);
	}
}

void View::closeEvent(QCloseEvent*)
{
//	kdDebug(66666) << k_funcinfo << "called." << endl;
	hide();
}

void View::showEvent(QShowEvent */*e*/)
{
//	kdDebug(66666) << k_funcinfo << "called." << endl;
//	KMainWindow::showEvent(e);
	QTimer::singleShot(0,this,SLOT(slotLoadWindowState()));
	emit shown();
}

void View::hideEvent(QHideEvent */*e*/)
{
//	kdDebug(66666) << k_funcinfo << "called." << endl;
//	KMainWindow::hideEvent(e);
	QTimer::singleShot(0,this,SLOT(slotSaveWindowState()));
	emit hidden();
}

void View::setModified(bool b)
{
	modified = b;
	if (!mPlaylistFile.isValid() || mPlaylistFile.fileName() == PLAYLISTFILE)
		setCaption(i18n("Playlist"), modified);
	else
		setCaption(i18n("%1 - Playlist").arg(mPlaylistFile.fileName()), modified);
}

void View::setModified()
{
	setModified(true);
}

void View::slotLoadWindowState(void)
{
	if (!FPL->prefs()->rememberGeom())
		return;
//	kdDebug(66666) << k_funcinfo << "called." << endl;
	KConfig *config = KGlobal::config();
	config->setGroup("General Options");

	QSize defaultSize(320,320);
	QPoint pos = config->readPointEntry("Position");
	QSize size = config->readSizeEntry("Geometry", &defaultSize);

	QRect geom (pos,size);
	QDesktopWidget* desk = QApplication::desktop();
	QRect deskRect = desk->screenGeometry(desk->screenNumber(pos));

	if(deskRect.contains(geom,true))
	{ // don't restore geometry if it's outside visible area
		move(pos);
		resize(size);
	}
}

void View::slotSaveWindowState()
{
	if (!FPL->prefs()->rememberGeom())
		return;
//	kdDebug(66666) << k_funcinfo << "called." << endl;
	KConfig *config = KGlobal::config();
	config->setGroup("General Options");

	config->writeEntry("Position", pos());
	config->writeEntry("Geometry", size());

	config->sync();
}

void View::saveState()
{
	//kdDebug(66666) << k_funcinfo << "BEGIN" << endl;
	FPL->prefs()->setModified(modified);
	FPL->prefs()->setFile(mPlaylistFile.path());
#if NOATUN_2_3
	saveToURL(napp->dirs()->saveLocation("data", "noatun/") + "flexplaylist.m3u");
#else
	saveToURL(napp->dirs()->saveLocation("data", "noatun/") + "flexplaylist.xml");
#endif

	int i;
	PlaylistItem item=FPL->getFirst();
	for(i = 0; item && item != FPL->current();)
		item=FPL->getAfter(item), i++;
	FPL->prefs()->setSaved(i);
	FPL->prefs()->save();

	KConfig *c = KGlobal::config();
	c->setGroup("General Options");
	c->writeEntry("Window Shown", !isHidden());

	saveMainWindowSettings(c, "FPL Window");

	//kdDebug(66666) << k_funcinfo << "END" << endl;
}

void View::configureToolBars()
{
	saveMainWindowSettings(KGlobal::config(), "FPL Window");
	KEditToolbar dlg(actionCollection(), "fplui.rc");
	connect(&dlg, SIGNAL(newToolbarConfig()), SLOT(newToolBarConfig()));
	dlg.exec();
}

void View::newToolBarConfig()
{
	createGUI("fplui.rc");
	applyMainWindowSettings(KGlobal::config(), "FPL Window");
}

// turns the sorting on or off
void View::setSorting(bool on, int column)
{
	if (on)
	{
		list->setSorting(column,true);
		list->setShowSortIndicator(true);
	}
	else
	{
		list->setShowSortIndicator(false);
		list->setSorting(-1);
	}
}

void View::headerClicked(int column)
{
	// this is to avoid that if we already have it sorted,
	// we sort it again ascendingly this way, clicking on
	// the header a second time will correctly toggle
	// ascending/descending sort
	if (list->showSortIndicator())
		return;
	else
		setSorting(true,column);
}

void View::slotSelectionChanged()
{
	bool on = (list->selectedItems().count()>0);
	mDelete->setEnabled(on);
	mPlayCurrent->setEnabled(on);
	mPlaySelectedNext->setEnabled(on);
	mItemProperties->setEnabled(on);
}

void View::slotReadConfig()
{
//	kdDebug(66666) << k_funcinfo << "called" << endl;

	list->setFont(FPL->prefs()->font());
	if(FPL->prefs()->showHeader())
		list->header()->show();
	else
		list->header()->hide();

	if(FPL->prefs()->showStatusButtons())
		mStatusBarWidget->show();
	else
		mStatusBarWidget->hide();

	list->setUpdatesEnabled(false);
	for (SafeListViewItem *i=static_cast<SafeListViewItem*>(listView()->firstChild());
	     i != 0; i=static_cast<SafeListViewItem*>(i->itemBelow()))
	{
		i->updatePixmap();
	}
	list->setUpdatesEnabled(true);
}

void View::slotConfKeys()
{
	KKeyDialog::configure(actionCollection(), this, true);
}

void View::slotOpenPrefs()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
#if NOATUN_2_3
	napp->preferencesBox()->show(FPL->prefs());
#else
	napp->preferencesBox()->show();
#endif
}

void View::updateStatusBar()
{
//	kdDebug(66666) << k_funcinfo << "Called" << endl;
	int secs = FPL->fullLength();
	int seconds, minutes, hours;
	QString lenString;

	seconds = secs % 60;
	minutes = (secs - seconds) / 60;
	hours = minutes / 60;
	minutes %= 60;

	lenString.sprintf("%d:%.2d:%.2d", hours, minutes, seconds);

	statusBar()->changeItem(i18n("List: %1").arg(lenString), STATUS_LISTLEN);

	if(FPL->current())
	{
		statusBar()->changeItem(i18n("Current: %1")
			.arg( (FPL->current()).lengthString() ),
			STATUS_CURLEN);
	}
	else
		statusBar()->changeItem(i18n("Current: 00:00"), STATUS_CURLEN);

//#ifdef FPL_DEBUG
	statusBar()->changeItem(i18n("Items: %1").arg(list->childCount()), STATUS_ITEMCOUNT);
//#endif
}


void View::slotDoJump(const PlaylistItem &i)
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;

	PlaylistItem item = FPL->getFirst();
	do
	{
		if (item == i)
		{
//			kdDebug(66666) << k_funcinfo << "PlaylistItem found, let's play" << endl;
			FPL->setCurrent(i, true);
			return;
		}
	} while(item = FPL->getAfter(item));
//	kdDebug(66666) << k_funcinfo << "PlaylistItem not found" << endl;

	KMessageBox::sorry(this,
		i18n("The selected item was removed from the playlist and cannot be jumped to."),
		i18n("Cannot jump to item"));
}

// ------------------------------------------------------------------------------------

TrayButton::TrayButton(QWidget* parent, const char* name)
	: QPushButton (parent, name)
{
	setBackgroundMode(PaletteBackground);
}

void TrayButton::drawButton(QPainter *p)
{
	QPixmap bg( size() );
	QPainter pbg;

	pbg.begin(&bg);
	pbg.fillRect(rect(), colorGroup().brush(QColorGroup::Background));
	if ( isDown() || isOn() )
	{	// Draw shapes to indicate the down state.
		pbg.setPen(Qt::black);
		pbg.drawLine(0, 0, width()-1, 0);
		pbg.drawLine(0, 0, 0, height()-1);
		pbg.setPen(colorGroup().light());
		pbg.drawLine(0, height()-1, width()-1, height()-1);
		pbg.drawLine(width()-1, 0, width()-1, height()-1);
	}
	pbg.end();

	p->drawPixmap(0,0,bg); // draw the background

	// draw icon
	if ( !pixmap()->isNull() )
	{
		QRect br(1, 1, width()-2, height()-2);
		int dx = (br.width()   - pixmap()->width())   / 2;
		int dy = (br.height()  - pixmap()->height())  / 2;

		p->drawPixmap(br.x() + dx, br.y() + dy, *pixmap());
	}
}

#include "view.moc"
// vim: set noet ts=4 sts=4 sw=4:
